@@ -41,6 +41,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <math.h>
 
 #include "mpg321.h"
 
@@ -160,6 +161,22 @@ char * modestring(enum mad_mode mode)
     }
 }
 
+char *versionstring(int flags)
+{
+	switch(flags&(MAD_FLAG_LSF_EXT | MAD_FLAG_MPEG_2_5_EXT))
+	{
+		case MAD_FLAG_LSF_EXT | MAD_FLAG_MPEG_2_5_EXT:
+			return "2.5";
+		case MAD_FLAG_LSF_EXT:
+			return "2.0";
+		case 0:
+			return "1.0";
+		default:
+			return "?";
+	}
+}
+
+
 char * modestringucase(enum mad_mode mode)
 {
     switch(mode)
@@ -214,6 +231,12 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
 
     mad_timer_add(&current_time, header->duration);
 
+    if(options.opt & MPG321_USE_SCROBBLER && scrobbler_time > 0 && scrobbler_time < current_time.seconds)
+    {
+	    scrobbler_time = -1;
+	    scrobbler_report();
+    }
+
     if(options.opt & (MPG321_VERBOSE_PLAY | MPG321_REMOTE_PLAY))
     {
         mad_timer_string(current_time, long_currenttime_str, "%.2u:%.2u.%.2u", MAD_UNITS_MINUTES,
@@ -246,7 +269,7 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
         file_change = 0;
         if (options.opt & MPG321_REMOTE_PLAY)
         {
-            printf("@S 1.0 %d %d %s %d %ld %d %d %d %d %ld %d\n", header->layer, header->samplerate,
+            printf("@S %s %d %d %s %d %ld %d %d %d %d %ld %d\n",versionstring(header->flags), header->layer, header->samplerate,
                 modestringucase(header->mode), header->mode_extension, 
                 (header->bitrate / 8 / 100) * mad_timer_count(header->duration, MAD_UNITS_CENTISECONDS),
                 MAD_NCHANNELS(header), header->flags & MAD_FLAG_COPYRIGHT ? 1 : 0, 
@@ -256,11 +279,11 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
 
         else if (options.opt & MPG321_VERBOSE_PLAY)/*zip it good*/
         {
-            fprintf(stderr, "MPEG 1.0, Layer: %s, Freq: %d, mode: %s, modext: %d, BPF : %ld\n"
+            fprintf(stderr, "MPEG %s, Layer: %s, Freq: %d, mode: %s, modext: %d, BPF : %ld\n"
                     "Channels: %d, copyright: %s, original: %s, CRC: %s, emphasis: %d.\n"
                     "Bitrate: %ld Kbits/s, Extension value: %d\n"
                     "Audio: 1:1 conversion, rate: %d, encoding: signed 16 bit, channels: %d\n",
-                    layerstring(header->layer), header->samplerate, modestringucase(header->mode), header->mode_extension, 
+                    versionstring(header->flags),layerstring(header->layer), header->samplerate, modestringucase(header->mode), header->mode_extension, 
                     (header->bitrate / 100) * mad_timer_count(header->duration, MAD_UNITS_CENTISECONDS),
                     MAD_NCHANNELS(header), header->flags & MAD_FLAG_COPYRIGHT ? "Yes" : "No",
                     header->flags & MAD_FLAG_ORIGINAL ? "Yes" : "No", header->flags & MAD_FLAG_PROTECTION ? "Yes" : "No",
@@ -270,8 +293,8 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
 
         else if (!(options.opt & MPG321_QUIET_PLAY))/*I love Joey*/
         {
-            fprintf(stderr, "MPEG 1.0 layer %s, %ld kbit/s, %d Hz %s\n",
-                layerstring(header->layer), header->bitrate/1000, header->samplerate, modestring(header->mode));
+            fprintf(stderr, "MPEG %s layer %s, %ld kbit/s, %d Hz %s\n",
+                versionstring(header->flags),layerstring(header->layer), header->bitrate/1000, header->samplerate, modestring(header->mode));
         }
     }
     
@@ -291,7 +314,7 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
     {
         if (!options.skip_printing_frames 
             || (options.skip_printing_frames && !(current_frame % options.skip_printing_frames)))
-            fprintf(stderr, "Frame# %5lu [%5lu], Time: %s [%s], \r", current_frame, 
+			fprintf(stdout, "Frame# %5lu [%5lu], Time: %s [%s], \r", current_frame, 
                     playbuf->num_frames > 0 ? playbuf->num_frames - current_frame : 0, long_currenttime_str, long_remaintime_str);
     }
     
@@ -730,6 +753,73 @@ signed long audio_linear_dither(unsigned int bits, mad_fixed_t sample,
   return output >> scalebits;
 }
 
+static void perform_and_print(char *samples,fft_state *state)
+{
+	static int xranges[] = {0, 1, 2, 3, 5, 7, 10, 14, 20, 28, 40, 54, 74, 101, 137, 187, 255};
+	static double fftout[FFT_BUFFER_SIZE / 2 +1];
+
+	printf("@I FFT:");
+	
+	fft_perform((sound_sample *)samples, fftout, state);
+
+	int i;
+	for(i = 0; i < 256; i++)
+	{
+		fftout[i] = (int)(sqrt(fftout[i + 1])) >> 8;
+	}
+
+	for(i = 0; i < 16; i++)
+	{
+		int val = 0;
+		int j;
+		for(j = xranges[i]; j < xranges[i + 1]; j++)
+		{
+			val += (fftout[j]) / 128;
+		}
+		if( val > 127) val = 127;
+		printf("%03i ", val);
+	}
+	printf("\n");
+}
+
+static int process_fft( char *data, int size)
+{
+	static fft_state *left_fftstate;
+	static int init=0;
+	static char buff_samples[1024];
+	static int buff_sample_count;
+
+	if(!((options.opt & MPG321_PRINT_FFT) && (options.opt & MPG321_REMOTE_PLAY))) return 0;
+
+	if(!init)
+	{
+		buff_sample_count = 0;
+		left_fftstate = fft_init();
+		init = 1;
+	}
+
+	if(buff_sample_count && 0)
+	{
+		memcpy(&buff_samples[buff_sample_count], data, 1024 - buff_sample_count);
+		data = &data[buff_sample_count];
+		size = size - buff_sample_count;
+		perform_and_print(buff_samples, left_fftstate);
+		buff_sample_count = 0;
+	}
+
+	int samples;
+	for( samples = 0; (samples + 1030) < size; samples += 1024)
+	{
+		perform_and_print(&data[samples],left_fftstate);
+	}
+
+	if(samples < size && 0)
+	{
+		memcpy(buff_samples, &data[samples], size - samples);
+		buff_sample_count = size - samples;
+	}
+}
+
 enum mad_flow output(void *data,
                      struct mad_header const *header,
                      struct mad_pcm *pcm)
@@ -764,6 +854,10 @@ enum mad_flow output(void *data,
         open_ao_playdevice(header);        
     }        
 
+    static int peak_rate = 0;
+    static unsigned short int peak = 0;
+
+
     if (pcm->channels == 2)
     {
         while (nsamples--)
@@ -771,6 +865,8 @@ enum mad_flow output(void *data,
             tempsample = mad_f_mul(*left_ch++, options.volume);
             sample = (signed int) audio_linear_dither(16, tempsample, &dither);
 
+	    peak = abs(sample) > peak ? abs(sample) : peak;
+
 #ifndef WORDS_BIGENDIAN
             *ptr++ = (unsigned char) (sample >> 0);
             *ptr++ = (unsigned char) (sample >> 8);
@@ -782,6 +878,8 @@ enum mad_flow output(void *data,
             tempsample = mad_f_mul(*right_ch++, options.volume);
             sample = (signed int) audio_linear_dither(16, tempsample, &dither);
 
+	    peak = abs(sample) > peak ? abs(sample) : peak;
+
 #ifndef WORDS_BIGENDIAN
             *ptr++ = (unsigned char) (sample >> 0);
             *ptr++ = (unsigned char) (sample >> 8);
@@ -790,7 +888,8 @@ enum mad_flow output(void *data,
             *ptr++ = (unsigned char) (sample >> 0);
 #endif
         }
-
+	
+	process_fft(stream, pcm->length * 4);
         ao_play(playdevice, stream, pcm->length * 4);
     }
     
@@ -800,6 +899,8 @@ enum mad_flow output(void *data,
         {
             tempsample = mad_f_mul(*left_ch++, options.volume);
             sample = (signed int) audio_linear_dither(16, tempsample, &dither);
+
+	    peak = abs(sample) > peak ? abs(sample) : peak;
             
             /* Just duplicate the sample across both channels. */
 #ifndef WORDS_BIGENDIAN
@@ -814,7 +915,7 @@ enum mad_flow output(void *data,
             *ptr++ = (unsigned char) (sample >> 0);
 #endif
         }
-
+	process_fft(stream, pcm->length * 4);
         ao_play(playdevice, stream, pcm->length * 4);
     }
         
@@ -824,6 +925,8 @@ enum mad_flow output(void *data,
         {
             tempsample = mad_f_mul(*left_ch++, options.volume);
             sample = (signed int) audio_linear_dither(16, tempsample, &dither);
+
+	    peak = abs(sample) > peak ? abs(sample) : peak;
             
 #ifndef WORDS_BIGENDIAN
             *ptr++ = (unsigned char) (sample >> 0);
@@ -833,7 +936,20 @@ enum mad_flow output(void *data,
             *ptr++ = (unsigned char) (sample >> 0);
 #endif
         }
-        ao_play(playdevice, stream, pcm->length * 2);
+
+	process_fft(stream, pcm->length * 2);
+	ao_play(playdevice, stream, pcm->length * 2);
+    }
+
+    if(options.opt & MPG321_REMOTE_PLAY && (options.opt & MPG321_PRINT_FFT))
+    {
+	    peak_rate = (peak_rate + 1) % 7;
+	    if(!peak_rate)
+	    {
+		    int peak_scale = (int)round(10 * (peak / 32768.0));
+		    printf("@I Peak:%d\n", peak_scale);
+		    peak = 0;
+	    }
     }
 
     return MAD_FLOW_CONTINUE;        
