@@ -1,7 +1,7 @@
 /*
     mpg321 - a fully free clone of mpg123.
     options.c: Copyright (C) 2001, 2002 Joe Drew
-    
+    	       Copyright (C) 2006, 2007, 2008, 2009, 2010 Nanakos Chrysostomos
     Originally based heavily upon:
     plaympeg - Sample MPEG player using the SMPEG library
     Copyright (C) 1999 Loki Entertainment Software
@@ -47,6 +47,9 @@ extern int auth_enable_var;
 /*Basic Authentication Arguments*/
 char *basic_auth = NULL;
 
+/*xterm title setting flag*/
+extern int set_xterm;
+
 void parse_options(int argc, char *argv[], playlist *pl)
 {
     struct option long_options[] =
@@ -103,7 +106,11 @@ void parse_options(int argc, char *argv[], playlist *pl)
         { "shuffle", 0, 0, 'z' },
         { "random", 0, 0, 'Z' },
         { "remote", 0, 0, 'R' },
+        { "set_xterm", 0, 0, 'x' },
         { "stereo", 0, 0, 'T' },
+        { "scrobbler", 0, 0, 'S' },
+        { "recursive", 0, 0, 'B' },
+        { "fft", 0, 0, 'F' },
             
         /* takes parameters */
         { "frames", 1, 0, 'n' },
@@ -123,7 +130,7 @@ void parse_options(int argc, char *argv[], playlist *pl)
 
     while ((c = getopt_long(argc, argv, 
                                 "OPLTNEI824cy01mCd:h:f:b:r:G:" /* unimplemented */
-                                "A:D:vqtsVHzZRo:n:@:k:w:a:g:l:p:u:U:",     /* implemented */
+                                "A:D:SBFvqtsVHzZRxo:n:@:k:w:a:g:l:p:u:U:",     /* implemented */
                         long_options, &option_index)) != -1)
     {            
         switch(c)
@@ -132,6 +139,9 @@ void parse_options(int argc, char *argv[], playlist *pl)
             case '2': case '4': case 'c': case 'y': case '0': case '1': case 'm': case 'C':
             case 'd': case 'h': case 'f': case 'b': 
                 break;
+	    case 'x':
+		set_xterm=1;
+		break;
 	    case 'p':
 		proxy_server = strdup(optarg);
 		proxy_enable = 1;
@@ -165,6 +175,7 @@ void parse_options(int argc, char *argv[], playlist *pl)
 
             case 'v':
                 options.opt |= MPG321_VERBOSE_PLAY;
+		scrobbler_set_verbose(1);
                 setvbuf(stdout, NULL, _IONBF, 0);
                 break;
             
@@ -216,6 +227,18 @@ void parse_options(int argc, char *argv[], playlist *pl)
             case 's':
                 options.opt |= MPG321_USE_STDOUT;
                 break;
+
+	    case 'S':
+		options.opt |= MPG321_USE_SCROBBLER;
+		break;
+
+	    case 'B':
+		options.opt |= MPG321_RECURSIVE_DIR;
+		break;
+
+	    case 'F':
+		options.opt |= MPG321_PRINT_FFT;
+		break;
                 
             case 'o':
                 if (strcmp(optarg, "alsa") == 0)
